@@ -21,11 +21,14 @@
 //!
 
 mod connected;
+
 use crate::parser::blk_file::BlkFile;
 use crate::parser::errors::{OpError, OpResult};
 use crate::parser::script::{evaluate_script, ScriptInfo};
 use crate::parser::tx_index::TxDB;
+use std::ops::Deref;
 use std::path::Path;
+use std::sync::Arc;
 // re-exports
 pub use crate::iter::{BlockIter, ConnectedBlockIter};
 pub use crate::parser::block_index::{BlockIndex, BlockIndexRecord};
@@ -47,15 +50,28 @@ pub fn parse_script(script_pub_key: &str) -> OpResult<ScriptInfo> {
     Ok(evaluate_script(&script, Network::Bitcoin))
 }
 
+pub struct InnerDB {
+    pub block_index: BlockIndex,
+    pub blk_file: BlkFile,
+    pub tx_db: TxDB,
+}
+
 ///
 /// This is the main struct of this crate!! Click and read the doc.
 ///
 /// All queries start from initializing `BitcoinDB`.
 ///
-pub struct BitcoinDB {
-    pub block_index: BlockIndex,
-    pub blk_file: BlkFile,
-    pub tx_db: TxDB,
+/// Note: This is an Arc wrap around `InnerDB`.
+///
+#[derive(Clone)]
+pub struct BitcoinDB(Arc<InnerDB>);
+
+impl Deref for BitcoinDB {
+    type Target = InnerDB;
+
+    fn deref(&self) -> &Self::Target {
+        self.0.deref()
+    }
 }
 
 impl BitcoinDB {
@@ -93,29 +109,44 @@ impl BitcoinDB {
         } else {
             TxDB::null()
         };
-        Ok(BitcoinDB {
+        let inner = InnerDB {
             block_index,
             blk_file: BlkFile::new(blk_path.as_path())?,
             tx_db,
-        })
+        };
+        Ok(BitcoinDB(Arc::new(inner)))
     }
 
     ///
     /// Get the maximum height found in block index.
     ///
-    /// It is not necessarily true that any height smaller than
-    /// `get_max_height()` can be used to query `get_block()`.
+    /// Note, not all blocks lower than this height have
+    /// been downloaded (different from `get_block_count()`).
     ///
-    /// This max_height is only the max height of block header
-    /// information. The actual block data might not have been
-    /// synced for querying. `get_block_header()`, `get_height_from_hash()`
-    /// `get_hash_from_height()`, will necessarily return valid
-    /// data, as long as height is smaller than `get_max_height()`.
+    /// Deprecated: use `get_block_count()`
     ///
+    #[deprecated(since = "1.2.6", note = "use `get_block_count()` instead")]
     pub fn get_max_height(&self) -> usize {
         self.block_index.records.len()
     }
 
+    ///
+    /// Get the maximum number of blocks downloaded.
+    ///
+    /// This API guarantee that block 0 to `get_block_count() - 1`
+    /// have been downloaded and available for query.
+    ///
+    pub fn get_block_count(&self) -> usize {
+        let records = self.block_index.records.len();
+        for h in 0..records {
+            // n_tx == 0 indicates that the block is not downloaded
+            if self.block_index.records.get(h).unwrap().n_tx == 0 {
+                return h;
+            }
+        }
+        return records;
+    }
+
     ///
     /// Get block header information.
     ///
@@ -134,14 +165,14 @@ impl BitcoinDB {
     /// // launch without reading txindex
     /// let db = BitcoinDB::new(path, false).unwrap();
     ///
-    /// let mut total_number: usize = 0;
+    /// let mut total_number_of_tx: usize = 0;
     ///
     /// // This computation should finish immediately. No Disk Access.
-    /// for i in 0..700000 {
+    /// for i in 0..db.get_block_count() {
     ///     let header = db.get_header(i).unwrap();
-    ///     total_number += header.n_tx as usize;
+    ///     total_number_of_tx += header.n_tx as usize;
     /// }
-    /// println!("total tx from block 0 to 700000: {}.", total_number);
+    /// println!("total number of transactions found on disk : {}.", total_number_of_tx);
     /// ```
     ///
     pub fn get_header(&self, height: usize) -> OpResult<&BlockIndexRecord> {
@@ -202,10 +233,15 @@ impl BitcoinDB {
         }
     }
 
+    ///
     /// Get a transaction by providing txid.
     ///
-    /// This function requires `txindex` to be set to `true`,
-    /// and `txindex=1` when running Bitcoin Core.
+    /// This function requires `txindex` to be set to `true` for `BitcoinDB`,
+    /// and requires that flag `txindex=1` has been enabled when
+    /// running Bitcoin Core.
+    ///
+    /// A transaction cannot be found using this function if it is
+    /// not yet indexed using `txindex`.
     ///
     /// # Example
     /// ```rust
@@ -232,6 +268,10 @@ impl BitcoinDB {
         if !self.tx_db.is_open() {
             return Err(OpError::from("TxDB not open"));
         }
+        // give special treatment for genesis transaction
+        if self.tx_db.is_genesis_tx(txid) {
+            return Ok(self.get_block::<Block>(0)?.txdata.swap_remove(0).into());
+        }
         let record = self.tx_db.get_tx_record(txid)?;
         let tx = self
             .blk_file
@@ -242,8 +282,12 @@ impl BitcoinDB {
     ///
     /// Get the height of the block containing a particular transaction.
     ///
-    /// This function requires `txindex` to be set to `true`,
-    /// and `txindex=1` when running Bitcoin Core.
+    /// This function requires `txindex` to be set to `true` for `BitcoinDB`,
+    /// and requires that flag `txindex=1` has been enabled when
+    /// running Bitcoin Core.
+    ///
+    /// A transaction cannot be found using this function if it is
+    /// not yet indexed using `txindex`.
     ///
     pub fn get_height_of_transaction(&self, txid: &Txid) -> OpResult<i32> {
         if !self.tx_db.is_open() {
@@ -268,9 +312,8 @@ impl BitcoinDB {
     /// read (i.e., when the max height in the database met).
     ///
     /// This is a very efficient implementation.
-    /// Iterating from height 600000 to 700000 should take less than
-    /// 30 minutes.
-    /// The performance bottleneck is likely to be disk IO.
+    /// Using SSD and intel core i7 (4 core, 8 threads)
+    /// Iterating from height 0 to 700000 takes about 10 minutes.
     ///
     /// # Example
     ///
@@ -318,15 +361,15 @@ impl BitcoinDB {
     /// Formats: `Block` / `FBlock` / `SBlock`.
     ///
     /// # Performance
+    ///
     /// This iterator is implemented to read the blocks in concurrency,
     /// but the result is still produced in the given order in `heights`.
     /// Results read are stored in a synced queue for `next()`
     /// to get.
     ///
     /// This is a very efficient implementation.
-    /// Iterating from height 600000 to 700000 should take less than
-    /// 30 minutes.
-    /// The performance bottleneck is likely to be disk IO.
+    /// Using SSD and intel core i7 (4 core, 8 threads)
+    /// Iterating from height 0 to 700000 takes about 10 minutes.
     ///
     /// ## Fails Fast
     ///
@@ -346,7 +389,7 @@ impl BitcoinDB {
     /// let some_heights = vec![3, 5, 7, 9];
     ///
     /// // iterate over blocks from 600000 to 700000
-    /// for block in db.iter_heights::<Block>(some_heights) {
+    /// for block in db.iter_heights::<Block>(some_heights.clone()) {
     ///     for tx in block.txdata {
     ///         println!("do something for this transaction");
     ///     }
